// State machine
// 
// Simplified code as example for a state machine.
//
// by feyd//godX.de

#include <string>
#include <stdio.h>

// State class waiting for "hello" followed by username "user" and password "mypassword"
// if everything was correct the state will reach "authenticated"
// if the chain is broken (in "hello" or "password" state) the state will enter "disconnected"
// after valid authentication the state machine will wait for "disconnect" keyword and enter the "disconnected" state

class CStateMachine
{
	enum {
		STATE_CONNECT = 0,
		STATE_USERNAME,
		STATE_PASSWORD,
		STATE_AUTHENTICATED,
		STATE_DISCONNECTED,
		STATE_MAX
	};
public:
	CStateMachine() : m_nCurrentState(STATE_CONNECT) {}
	virtual ~CStateMachine() {}

	void Input(const std::string& sInput)
	{
		switch(m_nCurrentState)
		{
			case STATE_CONNECT:
				if(sInput.compare("hello")==0)
				{
					m_nCurrentState = STATE_USERNAME;
				} else {
					m_nCurrentState = STATE_DISCONNECTED;
				}
				break;
			case STATE_USERNAME:
				// store username
				m_sUsername = sInput;
				m_nCurrentState = STATE_PASSWORD;
				break;
			case STATE_PASSWORD:
				// check username and password
				if(m_sUsername=="user" && sInput=="mypassword")
				{
					m_nCurrentState = STATE_AUTHENTICATED;
				} else {
					m_nCurrentState = STATE_DISCONNECTED;
				}
				break;
			case STATE_AUTHENTICATED:
				if(sInput.compare("disconnect")==0)
				{
					m_nCurrentState = STATE_DISCONNECTED;
				}
			case STATE_DISCONNECTED:
				break;
			default:;
		};
	}

	void Output()
	{
		switch(m_nCurrentState)
		{
			case STATE_CONNECT:			printf("CONNECT      "); break;
			case STATE_USERNAME:		printf("USERNAME     "); break;
			case STATE_PASSWORD:		printf("PASSWORD     "); break;
			case STATE_AUTHENTICATED:	printf("AUTHENTICATED"); break;
			case STATE_DISCONNECTED:	printf("DISCONNECTED "); break;
			default:					printf("Unknown State"); break;
		};
	}

	int GetCurrentState() const 
	{
		return m_nCurrentState;
	}
protected:
	int m_nCurrentState;

	std::string m_sUsername;
};

// Application entry point (from libc)
int main(int argc, const char* argv[])
{
	printf("State machine example...\r\n\r\n");

	{
		CStateMachine stateMachine;
		for(int n=1; n<argc; n++)
		{
			printf("    (%3d) %10s: ", n, argv[n]);
			stateMachine.Output();
			printf(" -> ");
			stateMachine.Input(argv[n]);
			stateMachine.Output();
			printf("\r\n");
		}
	}
}
